use std::{
    fs::OpenOptions,
    io::{BufWriter, Write},
    path::Path,
};

use crate::parse_deser::*;

use super::*;

pub fn create_compile_commands_content(
    name: &str,
    target: &Target,
    config: &Config,
) -> Result<String> {
    let mut ret = "[\n".to_string();

    let compiler_flags = target.compiler_flags.join(" ");

    let pwd = Path::new(".");
    let pwd = pwd.canonicalize().unwrap();
    let pwd = pwd.to_str().unwrap();

    if target.sources.len() == 0 {
        return Err(Error {
            kind: ErrorKind::NoSources,
            message: format!(
                "compile_commands_target is set to `{0}`, however `{0}` has no sources.",
                name
            ),
        });
    }

    for (i, source) in target.sources.iter().enumerate() {
        let new_file = source.replace("/", "-");
        let obj_name = format!("{}/{}/obj/{}.o", config.build_dir, name, new_file);

        ret.push_str(&format!(
            "  {{
    \"directory\": \"{0}\",
    \"command\": \"{1} {2} -o {3} -c {4}\",
    \"file\": \"{4}\"
  }}",
            pwd, target.compiler, compiler_flags, obj_name, source
        ));
        if i != target.sources.len() - 1 {
            ret.push(',');
        }
        ret.push('\n');
    }

    ret.push_str("]\n");

    Ok(ret)
}

pub fn write_compile_commands(filename: &str, content: &[u8]) -> std::io::Result<()> {
    let file = OpenOptions::new()
        .write(true)
        .truncate(true)
        .create(true)
        .open(filename)?;

    let mut writer = BufWriter::new(file);
    writer.write(content)?;
    Ok(())
}
