use crate::deser::*;

#[derive(Debug, Clone)]
pub struct TargetOptions<'a> {
    pub inherit: bool,
    pub inherit_from: &'a str,
    pub default: bool,
    pub depend_on: Vec<&'a str>,
    pub compile_cmd_fmt: &'a str,
    pub link_cmd_fmt: &'a str,
}

impl<'a> From<&'a DeserTargetOptions> for TargetOptions<'a> {
    fn from(value: &'a DeserTargetOptions) -> Self {
        let mut ret = Self::get_default();

        if let Some(v) = &value.inherit {
            ret.inherit = *v; // dereference the bool to copy it
        }

        if let Some(v) = &value.inherit_from {
            ret.inherit_from = v;
        }

        if let Some(v) = &value.default {
            ret.default = *v; // dereference the bool to copy it
        }

        if let Some(v) = &value.depend_on {
            for string in v {
                ret.depend_on.push(string);
            }
        }

        if let Some(v) = &value.compile_cmd_fmt {
            ret.compile_cmd_fmt = v;
        }

        if let Some(v) = &value.link_cmd_fmt {
            ret.link_cmd_fmt = v;
        }

        ret
    }
}

impl<'a> TargetOptions<'a> {
    pub fn get_default() -> Self {
        Self {
            inherit: true,
            inherit_from: "main",
            default: false,
            depend_on: vec![],
            compile_cmd_fmt:
                "{compiler} {compiler_flags} -MD -MF {depfile} -o {object} -c {source}",
            link_cmd_fmt: "{linker} {linker_flags} -o {outfile} {objects} {linker_libs}",
        }
    }
}
